@@ -16,9 +16,9 @@
 #include <glib/gi18n.h>
 #include <errno.h>
 
-#include <libiphone/libiphone.h>
-#include <libiphone/lockdown.h>
-#include <libiphone/afc.h>
+#include <libimobiledevice/libimobiledevice.h>
+#include <libimobiledevice/lockdown.h>
+#include <libimobiledevice/afc.h>
 
 #include "gvfsbackendafc.h"
 #include "gvfsjobopenforread.h"
@@ -38,6 +38,11 @@
 #define G_VFS_BACKEND_AFC_MAX_FILE_SIZE G_MAXINT64
 int g_blocksize = 4096; /* assume this is the default block size */
 
+/* AFC_E_INVALID_ARGUMENT was renamed between 0.9.7 and 1.0.0 */
+#ifndef AFC_E_INVALID_ARG
+#define AFC_E_INVALID_ARG AFC_E_INVALID_ARGUMENT
+#endif /* !AFC_E_INVALID_ARG */
+
 struct _GVfsBackendAfc {
   GVfsBackend backend;
 
@@ -46,7 +51,7 @@ struct _GVfsBackendAfc {
   char *model;
   gboolean connected;
 
-  iphone_device_t dev;
+  idevice_t dev;
   afc_client_t afc_cli;
 };
 
@@ -62,7 +67,7 @@ static struct afc_error_mapping afc_error_to_g_io_error[] = {
       { AFC_E_READ_ERROR               , G_IO_ERROR_NOT_DIRECTORY },
       { AFC_E_WRITE_ERROR              , G_IO_ERROR_FAILED },
       { AFC_E_UNKNOWN_PACKET_TYPE      , G_IO_ERROR_FAILED },
-      { AFC_E_INVALID_ARGUMENT         , G_IO_ERROR_INVALID_ARGUMENT },
+      { AFC_E_INVALID_ARG              , G_IO_ERROR_INVALID_ARGUMENT },
       { AFC_E_OBJECT_NOT_FOUND         , G_IO_ERROR_NOT_FOUND },
       { AFC_E_OBJECT_IS_DIR            , G_IO_ERROR_IS_DIRECTORY },
       { AFC_E_DIR_NOT_EMPTY            , G_IO_ERROR_NOT_EMPTY },
@@ -124,7 +129,7 @@ g_vfs_backend_afc_close_connection (GVfsBackendAfc *self)
       afc_client_free (self->afc_cli);
       g_free (self->model);
       self->model = NULL;
-      iphone_device_free (self->dev);
+      idevice_free (self->dev);
     }
   self->connected = FALSE;
 }
@@ -194,23 +199,23 @@ g_vfs_backend_lockdownd_check (lockdownd_error_t cond, GVfsJob *job)
 }
 
 static int
-g_vfs_backend_iphone_check (iphone_error_t cond, GVfsJob *job)
+g_vfs_backend_idevice_check (idevice_error_t cond, GVfsJob *job)
 {
-  if (G_LIKELY(cond == IPHONE_E_SUCCESS))
+  if (G_LIKELY(cond == IDEVICE_E_SUCCESS))
         return 0;
 
   switch (cond)
     {
-    case IPHONE_E_INVALID_ARG:
+    case IDEVICE_E_INVALID_ARG:
       g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
-                        _("iPhone Device Error: Invalid Argument"));
+                        _("libimobiledevice Error: Invalid Argument"));
       break;
-    case IPHONE_E_NO_DEVICE:
+    case IDEVICE_E_NO_DEVICE:
       g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                        _("iPhone Device Error: No device found. Make sure usbmuxd is set up correctly."));
+                        _("libimobiledevice Error: No device found. Make sure usbmuxd is set up correctly."));
     default:
       g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
-                        _("Unhandled iPhone Device error (%d)"), cond);
+                        _("Unhandled libimobiledevice error (%d)"), cond);
       break;
     }
 
@@ -218,12 +223,12 @@ g_vfs_backend_iphone_check (iphone_error_t cond, GVfsJob *job)
 }
 
 static void
-_iphone_event_cb (const iphone_event_t *event, void *user_data)
+_idevice_event_cb (const idevice_event_t *event, void *user_data)
 {
   GVfsBackendAfc *afc_backend = G_VFS_BACKEND_AFC (user_data);
 
   g_return_if_fail (afc_backend->uuid != NULL);
-  if (event->event != IPHONE_DEVICE_REMOVE)
+  if (event->event != IDEVICE_DEVICE_REMOVE)
     return;
   if (g_str_equal (event->uuid, afc_backend->uuid) == FALSE)
     return;
@@ -232,7 +237,7 @@ _iphone_event_cb (const iphone_event_t *event, void *user_data)
 
   g_vfs_backend_afc_close_connection (afc_backend);
 
-  iphone_event_unsubscribe ();
+  idevice_event_unsubscribe ();
 
   /* TODO: need a cleaner way to force unmount ourselves */
   exit (1);
@@ -249,11 +254,12 @@ g_vfs_backend_afc_mount (GVfsBackend *backend,
   const char *str;
   char *tmp;
   char *display_name;
-  int port, virtual_port;
+  guint16 port;
+  int virtual_port;
   GMountSpec *real_spec;
   GVfsBackendAfc *self;
   int retries;
-  iphone_error_t err;
+  idevice_error_t err;
   lockdownd_client_t lockdown_cli = NULL;
   char *camera_x_content_types[] = { "x-content/audio-player", "x-content/image-dcf", NULL};
   char *media_player_x_content_types[] = {"x-content/audio-player", NULL};
@@ -262,7 +268,7 @@ g_vfs_backend_afc_mount (GVfsBackend *backend,
   self = G_VFS_BACKEND_AFC(backend);
   self->connected = FALSE;
 
-  iphone_event_subscribe (_iphone_event_cb, self);
+  idevice_event_subscribe (_idevice_event_cb, self);
 
   /* setup afc */
 
@@ -322,16 +328,17 @@ g_vfs_backend_afc_mount (GVfsBackend *backend,
 
   retries = 0;
   do {
-      err = iphone_device_new(&self->dev, self->uuid);
-      if (err == IPHONE_E_SUCCESS)
+      err = idevice_new(&self->dev, self->uuid);
+      if (err == IDEVICE_E_SUCCESS)
           break;
       g_usleep (G_USEC_PER_SEC);
   } while (retries++ < 10);
 
-  if (G_UNLIKELY(g_vfs_backend_iphone_check(err, G_VFS_JOB(job))))
+  if (G_UNLIKELY(g_vfs_backend_idevice_check(err, G_VFS_JOB(job))))
     goto out_destroy_service;
-  if (G_UNLIKELY(g_vfs_backend_lockdownd_check (lockdownd_client_new (self->dev,
-                                                                      &lockdown_cli),
+  if (G_UNLIKELY(g_vfs_backend_lockdownd_check (lockdownd_client_new_with_handshake (self->dev,
+                                                                                     &lockdown_cli,
+                                                                                     "gvfsd-afc"),
                                                 G_VFS_JOB(job))))
     {
       goto out_destroy_dev;
@@ -406,7 +413,7 @@ out_destroy_lockdown:
   lockdownd_client_free (lockdown_cli);
 
 out_destroy_dev:
-  iphone_device_free (self->dev);
+  idevice_free (self->dev);
 
 out_destroy_service:
   g_free (self->service);
@@ -427,6 +434,44 @@ g_vfs_backend_afc_unmount (GVfsBackend *backend,
   g_vfs_job_succeeded (G_VFS_JOB(job));
 }
 
+static gboolean file_get_info (GVfsBackendAfc *backend, const char *path, GFileInfo *info);
+
+static gboolean
+is_directory (GVfsBackendAfc *backend,
+              const char *path)
+{
+  gboolean result = FALSE;
+  GFileInfo *info;
+
+  info = g_file_info_new();
+  if (file_get_info (backend, path, info))
+    {
+      if (g_file_info_get_file_type (info) == G_FILE_TYPE_DIRECTORY)
+        result = TRUE;
+    }
+
+  g_object_unref (info);
+  return result;
+}
+
+static gboolean
+is_regular (GVfsBackendAfc *backend,
+            const char *path)
+{
+  gboolean result = FALSE;
+  GFileInfo *info;
+
+  info = g_file_info_new();
+  if (file_get_info (backend, path, info))
+    {
+      if (g_file_info_get_file_type (info) == G_FILE_TYPE_REGULAR)
+	result = TRUE;
+    }
+
+  g_object_unref (info);
+  return result;
+}
+
 /* Callback to open an existing file for reading. */
 static void
 g_vfs_backend_afc_open_for_read (GVfsBackend *backend,
@@ -439,6 +484,22 @@ g_vfs_backend_afc_open_for_read (GVfsBackend *backend,
   self = G_VFS_BACKEND_AFC(backend);
   g_return_if_fail (self->connected);
 
+  if (is_directory (self, path))
+    {
+      g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
+                        G_IO_ERROR_IS_DIRECTORY,
+                        _("Can't open directory"));
+      return;
+    }
+
+  if (!is_regular (self, path))
+    {
+      g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
+                        G_IO_ERROR_NOT_FOUND,
+                        _("File doesn't exist"));
+      return;
+    }
+
   if (G_UNLIKELY(g_vfs_backend_afc_check (afc_file_open (self->afc_cli,
                                                          path, AFC_FOPEN_RDONLY, &fd),
                                           G_VFS_JOB(job))))
@@ -779,6 +840,7 @@ g_vfs_backend_afc_set_info_from_afcinfo (GVfsBackendAfc *self,
           else if (g_str_equal (afcinfo[i+1], "S_IFDIR"))
             {
               type = G_FILE_TYPE_DIRECTORY;
+              content_type = g_strdup ("inode/directory");
             }
           else if (g_str_equal (afcinfo[i+1], "S_IFLNK"))
             {
@@ -823,6 +885,15 @@ g_vfs_backend_afc_set_info_from_afcinfo (GVfsBackendAfc *self,
         }
     }
 
+  if (content_type == NULL)
+    content_type = g_content_type_guess (basename, NULL, 0, NULL);
+  
+  if (content_type)
+    {
+      g_file_info_set_content_type (info, content_type);
+      g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE, content_type);
+    }
+
   /* and set some additional info */
   g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_UNIX_UID, getuid ());
   g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_UNIX_GID, getgid ());
@@ -836,13 +907,10 @@ g_vfs_backend_afc_set_info_from_afcinfo (GVfsBackendAfc *self,
     {
       if (type == G_FILE_TYPE_DIRECTORY)
         {
-          content_type = g_strdup ("inode/directory");
           icon = g_themed_icon_new ("folder");
         }
       else
         {
-          if (content_type == NULL)
-            content_type = g_content_type_guess (basename, NULL, 0, NULL);
           if (content_type)
             {
               icon = g_content_type_get_icon (content_type);
@@ -851,12 +919,6 @@ g_vfs_backend_afc_set_info_from_afcinfo (GVfsBackendAfc *self,
             }
         }
 
-      if (content_type)
-        {
-          g_file_info_set_content_type (info, content_type);
-          g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE, content_type);
-        }
-
       if (icon == NULL)
         icon = g_themed_icon_new ("text-x-generic");
 
@@ -966,6 +1028,36 @@ g_vfs_backend_afc_set_info_from_afcinfo (GVfsBackendAfc *self,
     }
 }
 
+static gboolean
+file_get_info (GVfsBackendAfc *backend,
+               const char *path,
+               GFileInfo *info)
+{
+  char **afcinfo = NULL;
+  const char *basename, *ptr;
+  gboolean result = FALSE;
+
+  g_return_val_if_fail (backend->connected, result);
+  g_return_val_if_fail (info, result);
+
+  if (G_LIKELY(afc_get_file_info (backend->afc_cli, path, &afcinfo) == AFC_E_SUCCESS))
+    {
+      ptr = strrchr (path, '/');
+      if (ptr && ptr[1] != '\0')
+        basename = ptr + 1;
+      else
+        basename = path;
+
+      g_vfs_backend_afc_set_info_from_afcinfo (backend, info, afcinfo, basename, path, NULL, 0);
+      result = TRUE;
+    }
+
+  if (afcinfo)
+    g_strfreev(afcinfo);
+
+  return result;
+}
+
 /* Callback for iterating over a directory. */
 static void
 g_vfs_backend_afc_enumerate (GVfsBackend *backend,
@@ -1304,8 +1396,7 @@ g_vfs_backend_afc_init (GVfsBackendAfc *self)
   if (g_getenv ("GVFS_DEBUG") != NULL)
     {
       /* enable full debugging */
-      iphone_set_debug_level (1);
-      iphone_set_debug_mask (DBGMASK_ALL);
+      idevice_set_debug_level (1);
     }
 }
 
